@@ -1,9 +1,9 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright 2000 by PostgreSQL Global Development Group
+ * Copyright (c) 2000-2006, PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql-server/src/bin/psql/input.h,v 1.15 2001/11/05 17:46:31 momjian Exp $
+ * $PostgreSQL: pgsql/src/bin/psql/input.h,v 1.28 2006/06/11 23:06:00 tgl Exp $
  */
 #ifndef INPUT_H
 #define INPUT_H
@@ -12,32 +12,36 @@
  * If some other file needs to have access to readline/history, include this
  * file and save yourself all this work.
  *
- * USE_READLINE and USE_HISTORY are the definite pointers regarding existence or not.
+ * USE_READLINE is the definite pointers regarding existence or not.
  */
 #ifdef HAVE_LIBREADLINE
+#define USE_READLINE 1
 #if defined(HAVE_READLINE_READLINE_H)
 #include <readline/readline.h>
-#define USE_READLINE 1
+#elif defined(HAVE_EDITLINE_READLINE_H)
+#include <editline/readline.h>
 #elif defined(HAVE_READLINE_H)
 #include <readline.h>
-#define USE_READLINE 1
 #endif
-#endif
-
-#if defined(HAVE_HISTORY_FUNCTIONS)
 #if defined(HAVE_READLINE_HISTORY_H)
 #include <readline/history.h>
-#define USE_HISTORY 1
+#elif defined(HAVE_EDITLINE_HISTORY_H)
+#include <editline/history.h>
 #elif defined(HAVE_HISTORY_H)
 #include <history.h>
-#define USE_HISTORY 1
 #endif
 #endif
 
-char	   *gets_interactive(char *prompt);
+#include "pqexpbuffer.h"
+
+
+char	   *gets_interactive(const char *prompt);
 char	   *gets_fromFile(FILE *source);
 
 void		initializeInput(int flags);
-bool		saveHistory(char *fname);
+bool		saveHistory(char *fname, bool encodeFlag);
+
+void		pg_append_history(const char *s, PQExpBuffer history_buf);
+void		pg_send_history(PQExpBuffer history_buf);
 
 #endif   /* INPUT_H */
